using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace TextureDemo
{
	public class TextureDemoForm : System.Windows.Forms.Form
	{
		private Device device;
		private VertexBuffer vertexBuffer;
		private IndexBuffer indexBuffer;
		private const int ROWS = 42;
		private const int COLUMNS = 53;
		private Material material;
		private Texture texture;
		private DateTime lastTime = System.DateTime.Now;
		private bool animating = true;
		private double time = 0.0;

		private System.ComponentModel.Container components = null;

		public TextureDemoForm()
		{
			InitializeComponent();

			SetStyle(ControlStyles.AllPaintingInWmPaint|ControlStyles.Opaque, true);

			PresentParameters presentParams = new PresentParameters();
			presentParams.Windowed = true;
			presentParams.SwapEffect = SwapEffect.Discard;
			presentParams.EnableAutoDepthStencil = true;			
			presentParams.AutoDepthStencilFormat = DepthFormat.D24X8;
			device = new Device(0, DeviceType.Hardware, this, CreateFlags.HardwareVertexProcessing, presentParams);

			vertexBuffer = new VertexBuffer(typeof(CustomVertex.PositionNormalTextured), (ROWS+1)*(COLUMNS+1), device, Usage.WriteOnly, 0, Pool.Managed);
			CustomVertex.PositionNormalTextured[] vertexArray = (CustomVertex.PositionNormalTextured[]) vertexBuffer.Lock(0, LockFlags.None);
			for(int vertex = 0; vertex < (ROWS+1)*(COLUMNS+1); vertex++)
			{
				float x = 2.0f*(vertex % (COLUMNS+1))/(float)COLUMNS - 1.0f;
				float y = 2.0f*(vertex / (COLUMNS+1))/(float)ROWS - 1.0f;
				float z =  0.5f+0.2f*(float)Math.Sin(8.0*x);
				float nx = 0.2f*8.0f*(float)Math.Cos(8.0*x);
				float ny = 0.0f;
				float nz = -1.0f;
				float u = (vertex % (COLUMNS+1))/(float)COLUMNS;
				float v = (vertex / (COLUMNS+1))/(float)ROWS;
				vertexArray[vertex] = new CustomVertex.PositionNormalTextured(x, y, z, nx, ny, nz, u, v);
			}
			vertexBuffer.Unlock();

			indexBuffer = new IndexBuffer(typeof(short), 2*3*ROWS*COLUMNS, device, Usage.WriteOnly, Pool.Managed);
			short[] indexArray = (short[]) indexBuffer.Lock(0, LockFlags.None);
			for(int quad = 0; quad < ROWS*COLUMNS; quad++) // zaehlt die Vierecke (= je zwei Dreiecke) durch
			{
				// Es gibt pro Zeile einen Vertex mehr (COLUMNS + 1) als Vierecke!
				int quadBase = (quad % COLUMNS) + (quad / COLUMNS)*(COLUMNS + 1);
				indexArray[6*quad  ] = (short)(quadBase);
				indexArray[6*quad+1] = (short)(quadBase + COLUMNS + 1);
				indexArray[6*quad+2] = (short)(quadBase + COLUMNS + 2);
				indexArray[6*quad+3] = (short)(quadBase + COLUMNS + 2);
				indexArray[6*quad+4] = (short)(quadBase + 1);
				indexArray[6*quad+5] = (short)(quadBase);					
			}
			indexBuffer.Unlock();
				
			material = new Material();
			material.Emissive = Color.FromArgb(23, 23, 33);
			material.Ambient = Color.FromArgb(23, 23, 33);
			material.Diffuse = Color.FromArgb(240, 255, 240);
			material.Specular = Color.White;
			material.SpecularSharpness = 100.0f;
				
			texture = TextureLoader.FromFile(device, "../../rgba.tga"); // TGA, um auch Alphakanal zu haben
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
					texture.Dispose();
					device.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Vom Windows Form-Designer generierter Code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// TextureDemoForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(624, 502);
			this.Name = "TextureDemoForm";
			this.Text = "Texture Demo";

		}
		#endregion

		[STAThread]
		static void Main() 
		{
			Application.Run(new TextureDemoForm());
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			base.OnPaint (e);

			System.DateTime now = System.DateTime.Now;
			if(animating)
			{
				time += (now - lastTime).TotalSeconds; // knnte theoretisch an die Grenzen der Auflsung von double stoen
			}
			lastTime = now;
				
			device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, Color.Blue, 1.0f, 0);
			device.BeginScene();

			device.RenderState.CullMode = Cull.None;
				
			device.VertexFormat = CustomVertex.PositionNormalTextured.Format;
			device.SetStreamSource(0, vertexBuffer, 0);
			device.Indices = indexBuffer;
				
			device.Lights[0].Type = LightType.Directional;
			device.Lights[0].Diffuse = Color.Beige;
			device.Lights[0].Specular = Color.White;
			device.Lights[0].Direction = new Vector3(1.0f, -1.0f, 1.0f);
			device.Lights[0].Update();
			device.Lights[0].Enabled = true;

			device.Material = material;
			device.RenderState.NormalizeNormals = true;
			device.RenderState.Ambient = Color.DarkSlateBlue;
			device.RenderState.Lighting = true;
			device.RenderState.SpecularEnable = true;
				
			// Transparenz
			// device.RenderState.SourceBlend = Blend.SourceAlpha;				
			// device.RenderState.DestinationBlend = Blend.InvSourceAlpha;
			// device.RenderState.BlendOperation = BlendOperation.Add;
			// device.RenderState.AlphaBlendEnable = true;				
				
			device.SetTexture(0, texture);
			device.SamplerState[0].MagFilter = TextureFilter.Linear;
			device.SamplerState[0].MinFilter = TextureFilter.Linear;
			device.SamplerState[0].MipFilter = TextureFilter.Linear;	
				
			device.Transform.Projection = Matrix.PerspectiveFovLH(0.8f, ClientSize.Width/(float)ClientSize.Height, 0.1f, 50.0f);
			device.Transform.View = Matrix.LookAtLH(new Vector3(0.0f, 0.0f, -2.0f), new Vector3(), new Vector3(0.0f, 1.0f, 0.0f));
			device.Transform.World = Matrix.RotationX(1.2f)*Matrix.RotationY(0.8f*(float)time);

			device.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, (ROWS+1)*(COLUMNS+1), 0, 2*ROWS*COLUMNS);
				
			device.EndScene();
			device.Present();

			Invalidate();
		}

		protected override void OnKeyPress(KeyPressEventArgs e)
		{
			if(!e.Handled && e.KeyChar == ' ')
			{
				animating = !animating;
				e.Handled = true;
			}
			base.OnKeyPress (e);
		}

	}
}
